signature complex_rationalTheory =
sig
  type thm = Thm.thm

  (*  Definitions  *)
    val COMPLEX_ADD_curried_def : thm
    val COMPLEX_ADD_tupled_primitive_def : thm
    val COMPLEX_DIV_def : thm
    val COMPLEX_GE_curried_def : thm
    val COMPLEX_GE_tupled_primitive_def : thm
    val COMPLEX_GT_curried_def : thm
    val COMPLEX_GT_tupled_primitive_def : thm
    val COMPLEX_LE_curried_def : thm
    val COMPLEX_LE_tupled_primitive_def : thm
    val COMPLEX_LT_curried_def : thm
    val COMPLEX_LT_tupled_primitive_def : thm
    val COMPLEX_MULT_curried_def : thm
    val COMPLEX_MULT_tupled_primitive_def : thm
    val COMPLEX_NEG_def : thm
    val COMPLEX_RECIPROCAL_def : thm
    val COMPLEX_SUB_curried_def : thm
    val COMPLEX_SUB_tupled_primitive_def : thm
    val DIVIDES_def : thm
    val IS_INT_def : thm
    val com_0_def : thm
    val com_1_def : thm
    val complex_rational_TY_DEF : thm
    val complex_rational_case_def : thm
    val complex_rational_size_def : thm
    val reduce_def : thm
    val reduced_dnm_def : thm
    val reduced_nmr_def : thm

  (*  Theorems  *)
    val COMPLEX_ADD_def : thm
    val COMPLEX_ADD_ind : thm
    val COMPLEX_GE_def : thm
    val COMPLEX_GE_ind : thm
    val COMPLEX_GT_def : thm
    val COMPLEX_GT_ind : thm
    val COMPLEX_LE_def : thm
    val COMPLEX_LE_ind : thm
    val COMPLEX_LT_def : thm
    val COMPLEX_LT_ind : thm
    val COMPLEX_MULT_ASSOC : thm
    val COMPLEX_MULT_COMM : thm
    val COMPLEX_MULT_RID : thm
    val COMPLEX_MULT_def : thm
    val COMPLEX_MULT_ind : thm
    val COMPLEX_SUB_def : thm
    val COMPLEX_SUB_ind : thm
    val complex_rational_11 : thm
    val complex_rational_Axiom : thm
    val complex_rational_case_cong : thm
    val complex_rational_induction : thm
    val complex_rational_nchotomy : thm
    val datatype_complex_rational : thm

  val complex_rational_grammars : type_grammar.grammar * term_grammar.grammar
(*
   [rat] Parent theory of "complex_rational"

   [COMPLEX_ADD_curried_def]  Definition

      |- ∀x x1. x + x1 = COMPLEX_ADD_tupled (x,x1)

   [COMPLEX_ADD_tupled_primitive_def]  Definition

      |- COMPLEX_ADD_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_ADD_tupled a.
              case a of
                (com a1 b1,com a2 b2) => I (com (a1 + a2) (b1 + b2)))

   [COMPLEX_DIV_def]  Definition

      |- ∀a b. a / b = a * COMPLEX_RECIPROCAL b

   [COMPLEX_GE_curried_def]  Definition

      |- ∀x x1. x ≥ x1 ⇔ COMPLEX_GE_tupled (x,x1)

   [COMPLEX_GE_tupled_primitive_def]  Definition

      |- COMPLEX_GE_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_GE_tupled a.
              case a of
                (com ra ia,com rb ib) => I (ra > rb ∨ (ra = rb) ∧ ia ≥ ib))

   [COMPLEX_GT_curried_def]  Definition

      |- ∀x x1. x > x1 ⇔ COMPLEX_GT_tupled (x,x1)

   [COMPLEX_GT_tupled_primitive_def]  Definition

      |- COMPLEX_GT_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_GT_tupled a.
              case a of
                (com ra ia,com rb ib) => I (ra > rb ∨ (ra = rb) ∧ ia > ib))

   [COMPLEX_LE_curried_def]  Definition

      |- ∀x x1. x ≤ x1 ⇔ COMPLEX_LE_tupled (x,x1)

   [COMPLEX_LE_tupled_primitive_def]  Definition

      |- COMPLEX_LE_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_LE_tupled a.
              case a of
                (com ra ia,com rb ib) => I (ra < rb ∨ (ra = rb) ∧ ia ≤ ib))

   [COMPLEX_LT_curried_def]  Definition

      |- ∀x x1. x < x1 ⇔ COMPLEX_LT_tupled (x,x1)

   [COMPLEX_LT_tupled_primitive_def]  Definition

      |- COMPLEX_LT_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_LT_tupled a.
              case a of
                (com ra ia,com rb ib) => I (ra < rb ∨ (ra = rb) ∧ ia < ib))

   [COMPLEX_MULT_curried_def]  Definition

      |- ∀x x1. x * x1 = COMPLEX_MULT_tupled (x,x1)

   [COMPLEX_MULT_tupled_primitive_def]  Definition

      |- COMPLEX_MULT_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_MULT_tupled a.
              case a of
                (com a1 b1,com a2 b2) =>
                  I (com (a1 * a2 − b1 * b2) (b1 * a2 + a1 * b2)))

   [COMPLEX_NEG_def]  Definition

      |- ∀a. ¬a = com_0 − a

   [COMPLEX_RECIPROCAL_def]  Definition

      |- ∀a b.
           COMPLEX_RECIPROCAL (com a b) =
           com (a / (a * a + b * b)) (-b / (a * a + b * b))

   [COMPLEX_SUB_curried_def]  Definition

      |- ∀x x1. x − x1 = COMPLEX_SUB_tupled (x,x1)

   [COMPLEX_SUB_tupled_primitive_def]  Definition

      |- COMPLEX_SUB_tupled =
         WFREC (@R. WF R)
           (λCOMPLEX_SUB_tupled a.
              case a of
                (com a1 b1,com a2 b2) => I (com (a1 − a2) (b1 − b2)))

   [DIVIDES_def]  Definition

      |- ∀m n. DIVIDES m n ⇔ (Num (ABS n) MOD Num (ABS m) = 0)

   [IS_INT_def]  Definition

      |- ∀a b.
           IS_INT (com a b) ⇔ DIVIDES (rat_dnm a) (rat_nmr a) ∧ (b = rat_0)

   [com_0_def]  Definition

      |- com_0 = com rat_0 rat_0

   [com_1_def]  Definition

      |- com_1 = com rat_1 rat_0

   [complex_rational_TY_DEF]  Definition

      |- ∃rep.
           TYPE_DEFINITION
             (λa0'.
                ∀'complex_rational' .
                  (∀a0'.
                     (∃a0 a1.
                        a0' =
                        (λa0 a1.
                           ind_type$CONSTR 0 (a0,a1) (λn. ind_type$BOTTOM))
                          a0 a1) ⇒
                     'complex_rational' a0') ⇒
                  'complex_rational' a0') rep

   [complex_rational_case_def]  Definition

      |- ∀a0 a1 f. complex_rational_CASE (com a0 a1) f = f a0 a1

   [complex_rational_size_def]  Definition

      |- ∀a0 a1. complex_rational_size (com a0 a1) = 1

   [reduce_def]  Definition

      |- ∀x y.
           reduce (x,y) =
           (let n = &gcd (Num (ABS x)) (Num (ABS y)) in (x / n,y / n))

   [reduced_dnm_def]  Definition

      |- ∀r. reduced_dnm r = SND (reduce (rep_frac (rep_rat r)))

   [reduced_nmr_def]  Definition

      |- ∀r. reduced_nmr r = FST (reduce (rep_frac (rep_rat r)))

   [COMPLEX_ADD_def]  Theorem

      |- com a1 b1 + com a2 b2 = com (a1 + a2) (b1 + b2)

   [COMPLEX_ADD_ind]  Theorem

      |- ∀P. (∀a1 b1 a2 b2. P (com a1 b1) (com a2 b2)) ⇒ ∀v v1. P v v1

   [COMPLEX_GE_def]  Theorem

      |- com ra ia ≥ com rb ib ⇔ ra > rb ∨ (ra = rb) ∧ ia ≥ ib

   [COMPLEX_GE_ind]  Theorem

      |- ∀P. (∀ra ia rb ib. P (com ra ia) (com rb ib)) ⇒ ∀v v1. P v v1

   [COMPLEX_GT_def]  Theorem

      |- com ra ia > com rb ib ⇔ ra > rb ∨ (ra = rb) ∧ ia > ib

   [COMPLEX_GT_ind]  Theorem

      |- ∀P. (∀ra ia rb ib. P (com ra ia) (com rb ib)) ⇒ ∀v v1. P v v1

   [COMPLEX_LE_def]  Theorem

      |- com ra ia ≤ com rb ib ⇔ ra < rb ∨ (ra = rb) ∧ ia ≤ ib

   [COMPLEX_LE_ind]  Theorem

      |- ∀P. (∀ra ia rb ib. P (com ra ia) (com rb ib)) ⇒ ∀v v1. P v v1

   [COMPLEX_LT_def]  Theorem

      |- com ra ia < com rb ib ⇔ ra < rb ∨ (ra = rb) ∧ ia < ib

   [COMPLEX_LT_ind]  Theorem

      |- ∀P. (∀ra ia rb ib. P (com ra ia) (com rb ib)) ⇒ ∀v v1. P v v1

   [COMPLEX_MULT_ASSOC]  Theorem

      |- a * (b * c) = a * b * c

   [COMPLEX_MULT_COMM]  Theorem

      |- a * b = b * a

   [COMPLEX_MULT_RID]  Theorem

      |- a * com_1 = a

   [COMPLEX_MULT_def]  Theorem

      |- com a1 b1 * com a2 b2 =
         com (a1 * a2 − b1 * b2) (b1 * a2 + a1 * b2)

   [COMPLEX_MULT_ind]  Theorem

      |- ∀P. (∀a1 b1 a2 b2. P (com a1 b1) (com a2 b2)) ⇒ ∀v v1. P v v1

   [COMPLEX_SUB_def]  Theorem

      |- com a1 b1 − com a2 b2 = com (a1 − a2) (b1 − b2)

   [COMPLEX_SUB_ind]  Theorem

      |- ∀P. (∀a1 b1 a2 b2. P (com a1 b1) (com a2 b2)) ⇒ ∀v v1. P v v1

   [complex_rational_11]  Theorem

      |- ∀a0 a1 a0' a1'.
           (com a0 a1 = com a0' a1') ⇔ (a0 = a0') ∧ (a1 = a1')

   [complex_rational_Axiom]  Theorem

      |- ∀f. ∃fn. ∀a0 a1. fn (com a0 a1) = f a0 a1

   [complex_rational_case_cong]  Theorem

      |- ∀M M' f.
           (M = M') ∧ (∀a0 a1. (M' = com a0 a1) ⇒ (f a0 a1 = f' a0 a1)) ⇒
           (complex_rational_CASE M f = complex_rational_CASE M' f')

   [complex_rational_induction]  Theorem

      |- ∀P. (∀r r0. P (com r r0)) ⇒ ∀c. P c

   [complex_rational_nchotomy]  Theorem

      |- ∀cc. ∃r r0. cc = com r r0

   [datatype_complex_rational]  Theorem

      |- DATATYPE (complex_rational com)


*)
end
