;Proof script of alternating series error bound


(include-book "arithmetic-5/top" :dir :system)

(encapsulate
  (((n-term *) => *)   ; (n-term n) is the |nth term| in our series
   ((big-index) => *)) ; big-index is a constant
  (local (defun big-index ()
           0))
  (defthm natp-big-index
    (natp (big-index))
    :rule-classes :type-prescription)
  (local (defun n-term (n)
           (/ (+ 1 n))))
  (defthm positive-rationalp-n-term
    (implies (natp n)
             (and (rationalp (n-term n))
                  (< 0 (n-term n))))
    :rule-classes :type-prescription)
  (defthm n-term-decreases
    (implies (and (natp n)
                  (<= 0 n)
                  (<= (big-index) n))
             (< (n-term (+ 1 n)) ; not (+ n 1) -- need normal form for linear
                (n-term n)))
    :rule-classes :linear))


(defun minus-1 (n)
(declare (xargs :guard (natp n)))
  (if (evenp n) 1 -1))

(defun n-term-sum1 (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (if (zp n)
      (* (minus-1 m) (n-term m))
    (+ (* (minus-1 (+ m n)) (n-term (+ m n)))
       (n-term-sum1 m (- n 1)))))


(defthm n-term-sum1-split
  (implies (and (natp m)
                (natp n))
           (equal (n-term-sum1 (+ m 1) n)
                  (- (n-term-sum1 0 (+ m 1 n))
                     (n-term-sum1 0 m))))
 :hints (("Goal" :Induct (n-term-sum1 m n)))
)

(defun n-terms-pair1 (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (if (zp n)
      (n-term m)
    (- (n-terms-pair1 m (- n 1))
       (- (n-term (+ m (- (* 2 n) 1)))
          (n-term (+ m (* 2 n)))))))

#||(defthm n-term-sum1-split1
  (implies (and (natp m)
                (natp n))
           (equal (n-term-sum1 (+ (* 2 m) 1) (* 2 n))
                (- (n-term-sum1 0 (+ (* 2 m) 1 (* 2 n)))
                   (n-term-sum1 0 (* 2 m)))))
:hints (("Goal" ;:Induct (n-term-sum1 (+ (* 2 m) 1) (* 2 n))
         :in-theory (disable n-term-sum1-split)
         :use ((:instance n-term-sum1-split
                          (m (* 2 m))
                          (n (* 2 n))))
))

)||#

(defthm n-term-sum1-split1
  (implies (and (natp m)
                (natp n))
           (equal (- (n-term-sum1 0 (+ m 1 (* 2 n)))
                     (n-term-sum1 0 m))
                  (n-term-sum1 (+ m 1) (* 2 n))))
  :hints (("Goal" ;:Induct (n-term-sum1 (+ (* 2 m) 1) (* 2 n))
           :in-theory (disable n-term-sum1-split)
           :use ((:instance n-term-sum1-split
                            (m m)
                            (n (* 2 n))))))
)


(defthm n-term-sum1-split2
  (implies (and (natp m)
                (natp n))
           (equal (- (n-term-sum1 0 (+ (* 2 m) 1 (* 2 n)))
                     (n-term-sum1 0 (* 2 m)))
                  (n-term-sum1 (+ (* 2 m) 1) (* 2 n))))
  :hints (("Goal" ;:Induct (n-term-sum1 (+ (* 2 m) 1) (* 2 n))
           :in-theory (disable n-term-sum1-split)
           :use ((:instance n-term-sum1-split
                            (m (* 2 m))
                            (n (* 2 n))))))

  )

#||(defthm n-term-sum-eq-n-terms-pair
  (implies (and (natp m)
                (natp n))
           (equal (n-term-sum m (* 2 n))
                  (* (minus-1 m) (n-terms-pair m n))))
  :hints (("Goal"
           :Induct (n-terms-pair m n)
           :expand ((n-term-sum m (* 2 n))))))||#

(defthm n-terms-pair-lt-n-term1
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (n-terms-pair1 m n)
               (n-term m)))
 :hints (("Goal" :Induct (n-terms-pair1 m n)
           :in-theory (disable POSITIVE-RATIONALP-N-TERM))
          ("Subgoal *1/2"
           :use ((:instance n-term-decreases
                            (n (+ -1 m (* 2 n))))
                 (:instance n-term-decreases
                            (n m))
                 (:instance positive-rationalp-n-term
                            (n m))))) 
)

(defthm n-terms-pair-lt-n-term2
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (-(n-terms-pair1 m n)
                 (n-term (+ m (* 2 n) 1)))
               (n-term m)))
  :hints (("Goal"
           :in-theory (disable n-terms-pair-lt-n-term1)
           :use n-terms-pair-lt-n-term1)))

(defun n-terms-pair-rev (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (if (zp n)
      0
    (+ (n-terms-pair-rev m (- n 1))
       (- (n-term (+ m (- (* 2 n) 2)))
          (n-term (+ m (- (* 2 n) 1)))))))

(defun n-terms-pair-rev-total (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (+ (n-terms-pair-rev m n)
     (n-term  (+ m (* 2 n)))))

(defthm n-term-sum1-eq-n-terms-pair-rev-lemma
  (implies (and (natp m)
                (natp n))
           (equal (n-term-sum1 m (* 2 n))
                  (* (minus-1 m)
                       (n-terms-pair-rev-total m n))))
  :hints (("Goal" :Induct (n-terms-pair-rev m n)
           ;:in-theory (disable n-term-sum-eq-n-terms-pair1)
           :expand (n-term-sum1 m (* 2 n)))))


(defthm n-term-sum1-eq-n-terms-pair-rev
  (implies (and (natp m)
                (natp n))
           (equal  (n-term-sum1 m (* 2 n))
                   (* (minus-1 m)
                      (+ (n-terms-pair-rev m n)
                         (n-term (+ m (* 2 n)))))))
  :hints (("Goal" ;:Induct (n-terms-pair-rev m n)
           ;:in-theory (disable n-term-sum-eq-n-terms-pair1)
           :expand (n-term-sum1 m (* 2 n)))))

(defthm n-term-sum1-abs-eq-n-term-pair-rev-abs
  (implies (and (natp m)
                (natp n))
           (equal  (abs (n-term-sum1 m (* 2 n)))
                   (abs
                      (+ (n-terms-pair-rev m n)
                         (n-term (+ m (* 2 n)))))))
  :hints (("Goal" ;:Induct (n-terms-pair-rev m n)
           ;:in-theory (disable n-term-sum-eq-n-terms-pair1)
           :expand (n-term-sum1 m (* 2 n)))))



(defthm n-terms-pair-rev-eq-n-terms-pair
  (implies (and (natp m)
                (natp n))
           (equal (+ (n-terms-pair-rev m n)
                     (n-term (+ m (* 2 n ))))
                  (n-terms-pair1 m n)))
  :hints (("Goal" :Induct (n-terms-pair1 m n))))

(defthm n-terms-pair-rev-pos-lemma
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= 0
               (n-terms-pair-rev m n)))
  :hints (("Goal" 
           :induct (n-terms-pair-rev m n)
           :in-theory (disable positive-rationalp-n-term))
          ("Subgoal *1/2"
           :use ((:instance n-term-decreases
                            (n (+ m -2 (* 2 n))))
                 (:instance positive-rationalp-n-term
                            (n m))))))

(defthm n-terms-pair-rev-pos-lemma-1
  (implies (and (natp m)
                (natp n))
           (<= 0
               (n-term (+ m 1 (* 2 n))))))

(defthm n-terms-pair-rev-pos
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= 0
               (+ (n-terms-pair-rev m n)
                  (n-term (+ m 1 (* 2 n))))))
  :hints (("Goal"
           :in-theory (disable n-terms-pair-rev-pos-lemma)
           :use ((:instance n-terms-pair-rev-pos-lemma )))))


(defthm abs-n-term-sum-lt-n-term
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs (n-term-sum1 m (* 2 n)))
               (n-term m)))
  :hints (("Goal"  
           :in-theory (disable n-terms-pair-rev-eq-n-terms-pair)
           :use ((:instance n-terms-pair-rev-pos)
                 (:instance n-terms-pair-rev-eq-n-terms-pair)))))

(defthm abs-n-term-sum-split1-lt-n-term
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs(- (n-term-sum1 0 (+ (* 2 m) 1 (* 2 n)))
                      (n-term-sum1 0 (* 2 m))))
               (n-term (+ (* 2 m) 1))))
  :hints (("Goal"   
           :in-theory (disable n-term-sum1-abs-eq-n-term-pair-rev-abs
                               n-term-sum1-eq-n-terms-pair-rev
                               n-term-sum1-eq-n-terms-pair-rev-lemma)
           :use ((:instance n-term-sum1-split2 )
                 (:instance abs-n-term-sum-lt-n-term
                            (m (+ 1 (* 2 m))))))))



(defthm abs-n-term-sum-split1-lt-n-term1
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs(- (n-term-sum1 0 (+ m 1 (* 2 n)))
                      (n-term-sum1 0 m)))
               (n-term (+ m 1))))
  :hints (("Goal"   
           :in-theory (disable n-term-sum1-abs-eq-n-term-pair-rev-abs
                               n-term-sum1-eq-n-terms-pair-rev
                               n-term-sum1-eq-n-terms-pair-rev-lemma)
           :use ((:instance n-term-sum1-split1)
                 (:instance abs-n-term-sum-lt-n-term
                            (m (+ m 1)))))))

;When terms are odd beyond a_m

(defthm n-term-sum1-split3
  (implies (and (natp m)
                (natp n))
           (equal (- (n-term-sum1 0 (+ m 1 (* 2 n) 1))
                     (n-term-sum1 0 m))
                  (n-term-sum1 (+ m 1) (+ (* 2 n) 1))))
  :hints (("Goal" 
           :in-theory (disable n-term-sum1-split)
           :use ((:instance n-term-sum1-split
                            (m m)
                            (n (+ (* 2 n) 1)))))))

(defun n-terms-pair2 (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (- (n-terms-pair1 m n)
     (n-term (+ m (+ (* 2 n) 1)))))

(defthm n-terms-pair-lt-n-term3
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (n-terms-pair2 m n)
               (n-term m)))
  :hints (("Goal"
           :in-theory (disable n-terms-pair-lt-n-term1)
           :use n-terms-pair-lt-n-term1)))

(defun n-terms-pair-rev1 (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (if (zp n)
      0
    (+ (n-terms-pair-rev1 m (- n 1))
       (- (n-term (+ m (- (* 2 n) 2)))
          (n-term (+ m (- (* 2 n) 1)))))))

(defun n-terms-pair-rev1-total (m n)
  (declare (xargs :guard (and (natp n) (natp m)) :verify-guards nil))
  (+ (n-terms-pair-rev1 m n)
     (- (n-term (+ m (* 2 n)))
        (n-term (+ m (+ (* 2 n) 1))))))

(defthm n-term-sum1-eq-n-terms-pair-rev-lemma1
  (implies (and (natp m)
                (natp n))
           (equal (n-term-sum1 m (+ (* 2 n) 1))
                  (* (minus-1 m)
                       (n-terms-pair-rev1-total m n))))
  :hints (("Goal" 
           :induct (n-terms-pair-rev1 m n)
           :in-theory (disable n-term-sum1-eq-n-terms-pair-rev
                               n-term-sum1-eq-n-terms-pair-rev-lemma)
           :expand (n-term-sum1 m (* 2 n)))))

(defthm n-term-sum1-abs-eq-n-term-pair-rev1-abs
  (implies (and (natp m)
                (natp n))
           (equal  (abs (n-term-sum1 m (+ (* 2 n) 1)))
                   (abs (n-terms-pair-rev1-total m n))))
  :hints (("Goal"
           :in-theory (disable n-term-sum1-eq-n-terms-pair-rev-lemma1)
           :use (n-term-sum1-eq-n-terms-pair-rev-lemma1))))


(defthm n-terms-pair-rev-eq-n-terms-pair1
  (implies (and (natp m)
                (natp n))
           (equal (n-terms-pair-rev1-total m n)
                  (n-terms-pair2 m n)))
  :hints (("Goal" 
           :induct (n-terms-pair1 m n))))

(defthm n-terms-pair-rev-pos-lemma1
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= 0
               (n-terms-pair-rev1 m n)))
  :hints (("Goal"
           :induct (n-terms-pair-rev1 m n)
           :in-theory (disable positive-rationalp-n-term))
          ("Subgoal *1/2"
           :use ((:instance n-term-decreases
                            (n (+ m -2 (* 2 n))))
                 (:instance positive-rationalp-n-term
                            (n m))))))

(defthm n-terms-pair-rev-pos-lemma-2
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= 0
               (- (n-term (+ m (* 2 n)))
                  (n-term (+ m (+ (* 2 n) 1))))))
  :hints (("Goal"
           :use ((:instance n-term-decreases
                            (n (+ m 1 (* 2 n))))))))

(defthm n-terms-pair-rev-pos1
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= 0
               (+ (n-terms-pair-rev1 m n)
                  (- (n-term (+ m (* 2 n)))
                     (n-term (+ m (+ (* 2 n) 1)))))))
  :hints (("Goal"
           :in-theory (disable n-terms-pair-rev-pos-lemma1
                               n-terms-pair-rev-pos-lemma-2)
           :use ((:instance n-terms-pair-rev-pos-lemma1)
                 (:instance n-terms-pair-rev-pos-lemma-2)))))

(defthm n-terms-pair-rev-pos2
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= 0 (n-terms-pair-rev1-total m n)))
  :hints (("Goal"
           :in-theory (disable n-terms-pair-rev-pos1
                               n-terms-pair-rev-eq-n-terms-pair1)
           :use ((:instance n-terms-pair-rev-pos1)))))



(defthm abs-n-term-sum-lt-n-term1
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs(n-term-sum1 m (+ (* 2 n) 1)))
               (n-term m)))
  :hints (("Goal"
           :in-theory (disable n-term-sum1-abs-eq-n-term-pair-rev1-abs
                               n-terms-pair-rev-pos2
                               n-terms-pair-lt-n-term2)
           :use ((:instance n-term-sum1-abs-eq-n-term-pair-rev1-abs)
                 (:instance n-terms-pair-rev-pos2)
                 (:instance n-terms-pair-lt-n-term2)))))


(defthm abs-n-term-sum-split1-lt-n-term2
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs (- (n-term-sum1 0 (+ m 1 (* 2 n) 1))
                       (n-term-sum1 0 m)))
               (n-term (+ m 1))))
  :hints (("Goal"
           :in-theory (disable n-term-sum1-abs-eq-n-term-pair-rev1-abs)
           :use ((:instance n-term-sum1-split3)
                 (:instance abs-n-term-sum-lt-n-term1
                            (m (+ m 1)))))))


(defthm abs-n-term-sum-split1-lt-n-term3
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs (- (n-term-sum1 0 (+ m 1 n))
                       (n-term-sum1 0 m)))
               (n-term (+ m 1))))
  :hints (("Goal"
           :do-not-induct t
           :use ((:instance abs-n-term-sum-split1-lt-n-term1
                            (n (/ n 2)))
                 (:instance abs-n-term-sum-split1-lt-n-term2
                            (n (/ (1- n) 2)))))))


(defthm abs-n-term-sum-split1-lt-n-term4
  (implies (and (natp m)
                (<= (big-index) m)
                (natp n))
           (<= (abs (n-term-sum1 (+ m 1) n))
               (n-term (+ m 1))))
  :hints (("Goal"
           :do-not-induct t
           :use ((:instance n-term-sum1-split)
                 (:instance abs-n-term-sum-split1-lt-n-term3)))))


