
(* ========================================================================= *)
(* File Name: medical_diagScript.sml	                                     *)
(*---------------------------------------------------------------------------*)
(* Description: Formal Reliability Analysis of Medical Diagonistic System    *)
(*           Using   Higher-order Logic                                      *)
(*                                                                           *)
(*                HOL4-Kananaskis 10 		 			     *)
(*									     *)
(*		Author :  Waqar Ahmad             		     	     *)
(*                                              			     *)
(* 	    School of Electrical Engineering and Computer Sciences (SEECS)   *)
(*	    National University of Sciences and Technology (NUST), PAKISTAN  *)
(*                                          		               	     *)
(*                                                                           *)
(* ========================================================================= *)

   
(*-----------------------------*)
(* Medical Case Study          *)
(*-----------------------------*)
(*loadPath := "/home/waqar/Downloads/acl2-link/acl2-link/HOL-to-ACL2-exp-function" :: !loadPath; *)
loadPath := "/home/waqar/Downloads/HOL-Code-thesis/Formal-Dependability" :: !loadPath;
(*loadPath := "/home/waqar/Downloads/acl2-link/acl2-link/ml" :: !loadPath;*)
app load ["RBDTheory", "VDCTheory","extrealTheory","probabilityTheory","realTheory","realLib","arithmeticTheory","ratTheory","integerTheory","ratLib","intrealTheory","transcTheory","combinTheory","pred_setTheory","util_probTheory","listTheory"];
open RBDTheory VDCTheory extrealTheory probabilityTheory realTheory realLib arithmeticTheory ratTheory integerTheory ratLib intrealTheory transcTheory combinTheory pred_setTheory util_probTheory listTheory;

(*--------------------*)
infixr 0 ++ << || ORELSEC ## --> THENC;
infix 1 >> |->;
fun parse_with_goal t (asms, g) =
  let
    val ctxt = free_varsl (g::asms)
  in
    Parse.parse_in_context ctxt t
  end;

val PARSE_TAC = fn tac => fn q => W (tac o parse_with_goal q);
val Suff = PARSE_TAC SUFF_TAC;
val POP_ORW = POP_ASSUM (fn thm => ONCE_REWRITE_TAC [thm]);
val !! = REPEAT;
val op++ = op THEN;
val op<< = op THENL;
val op|| = op ORELSE;
val op>> = op THEN1;
val std_ss' = simpLib.++ (std_ss, boolSimps.ETA_ss);



(*--------------------real_of_rat----------------------------*)
val real_of_rat = Define 
`real_of_rat a = 
real_of_int (reduced_nmr a) / real_of_int (reduced_dnm a) `;

(*------------------------exp_macl_dist_def-------------------*)
val exp_macl_dist_def = Define 
`exp_macl_dist p X l m n = 
 !t. (CDF p X (real_of_rat t) = 
     if (0 <= real_of_rat t) then
     	  1 - real_of_rat (expt_macl (l* t) m n) else (0:real))`;
(*-----------------exp_macl_dist_list_def---------------------*)
val exp_macl_dist_list_def = Define 
`(exp_macl_dist_list p [] L m n = T) /\
 (exp_macl_dist_list p (h::t) L m n =
 (exp_macl_dist p h (HD L) m n) /\ (exp_macl_dist_list p t (TL L) m n))`;
(*--------------------exp_macl_func_list_def------------------*)
val exp_macl_func_list_def = Define 
`exp_macl_func_list C s m n = 
 MAP (\a. real_of_rat (expt_macl (a*s) m n)) C`;

(*--------------rel_med_lemma1-------------------------*)
val rel_med_lemma1 = store_thm("rel_med_lemma1",
  ``!a b t p m n. prob_space p /\
       	      (0 <= real_of_rat t) /\
	      (fail_event p a (real_of_rat t) IN events p) /\
	      exp_macl_dist p a b m n ==>
	      (prob p
              	  (PREIMAGE a {y | Normal (real_of_rat t) < y} ∩
               	   p_space p) =
	       real_of_rat (expt_macl (b * t) m n))``,
RW_TAC std_ss[exp_macl_dist_def,CDF_def,distribution_def]
++ FIRST_X_ASSUM (Q.SPECL_THEN [`t'`] MP_TAC)
++ RW_TAC std_ss[]
++ DEP_REWRITE_TAC[GSYM rel_event_def]
++ DEP_REWRITE_TAC[GSYM compl_fail_event_eq_rel_event]
++ DEP_REWRITE_TAC[PROB_COMPL]
++ RW_TAC std_ss[]
++ RW_TAC std_ss[real_sub]
++ RW_TAC std_ss[GSYM REAL_EQ_SUB_LADD]
++ RW_TAC std_ss[REAL_SUB_RNEG]
++ ONCE_REWRITE_TAC[REAL_ADD_COMM]
++ MATCH_MP_TAC EQ_SYM
++ RW_TAC std_ss[GSYM REAL_EQ_SUB_LADD]
++ RW_TAC std_ss[fail_event_def]);

(*---------------Medical Diagonistic Case Study---------------*)

val rel_medical_diag = store_thm("rel_medical_diag",
  ``!p m n s ER'' AC'' Ga'' EG'' Co'' DB'' CT'' CaA'' NS'' Inj1'' Inj2'' Inj3'' HP'' C'' Bd'' IR'' S'' MT'' C_ER C_AC C_Ga C_EG C_Co C_DB C_CT C_CaA C_NS C_Inj1 C_Inj2 C_Inj3 C_HP C_C C_Bd C_IR C_S C_MT.
 prob_space p /\ (0 <= real_of_rat s) /\
(!x'. MEM x' 
      (FLAT
         (two_dim_rel_event_list p
            [[ER''; AC''; Ga'']; [EG''; Co''; DB''; CT''; CaA''; NS''];
             [Inj1''; Inj2''; Inj3''; HP''; C''; Bd''; IR''; S''; MT'']]
            (real_of_rat s))) ==> (x' IN events p)) /\
(mutual_indep p 
  (FLAT
      (two_dim_rel_event_list p
         [[ER''; AC''; Ga'']; [EG''; Co''; DB''; CT''; CaA''; NS''];
          [Inj1''; Inj2''; Inj3''; HP''; C''; Bd''; IR''; S''; MT'']]
         (real_of_rat s)))) /\
(exp_macl_dist_list p ([ER'';AC'';Ga'';EG'';Co'';DB'';CT'';CaA'';NS'';Inj1'';Inj2'';Inj3'';HP'';C'';Bd'';IR'';S'';MT'']) ([C_ER;C_AC;C_Ga;C_EG;C_Co;C_DB;C_CT;C_CaA;C_NS;C_Inj1;C_Inj2;C_Inj3;C_HP;C_C;C_Bd;C_IR;C_S;C_MT]) m n) ==>
(prob p
        (rbd_struct p
           ((series of (\a. parallel (rbd_list a)))
              (two_dim_rel_event_list p [([(ER'':'a->extreal);AC'';Ga'']:('a->extreal) list);[EG'';Co'';DB'';CT'';CaA'';NS''];[Inj1'';Inj2'';Inj3'';HP'';C'';Bd'';IR'';S'';MT'']] (real_of_rat s) ))) = 
(list_prod of
       (\a. 1 − list_prod (one_minus_list (exp_macl_func_list (a) s m n))))
        ([[C_ER;C_AC;C_Ga];[C_EG;C_Co;C_DB;C_CT;C_CaA;C_NS];[C_Inj1;C_Inj2;C_Inj3;C_HP;C_C;C_Bd;C_IR;C_S;C_MT]]))``,
RW_TAC std_ss[]
THEN DEP_REWRITE_TAC[series_parallel_struct_thm_v2]
THEN RW_TAC list_ss[]
THEN1 FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def]
THEN RW_TAC list_ss[of_DEF,o_THM,two_dim_rel_event_list_def,rel_event_list_def]
THEN RW_TAC list_ss[list_prob_def,one_minus_list_def,list_prod_def,exp_macl_func_list_def]
THEN FULL_SIMP_TAC list_ss[exp_macl_dist_list_def]
THEN MP_TAC (Q.ISPECL [`ER'':'a->extreal`,`C_ER:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p ER'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
THEN MP_TAC (Q.ISPECL [`AC'':'a->extreal`,`C_AC:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p AC'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`Ga'':'a->extreal`,`C_Ga:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p Ga'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`EG'':'a->extreal`,`C_EG:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p EG'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`Co'':'a->extreal`,`C_Co:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p Co'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`DB'':'a->extreal`,`C_DB:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p DB'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`CT'':'a->extreal`,`C_CT:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p CT'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`CaA'':'a->extreal`,`C_CaA:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p CaA'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`NS'':'a->extreal`,`C_NS:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p NS'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`Inj1'':'a->extreal`,`C_Inj1:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p Inj1'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`Inj2'':'a->extreal`,`C_Inj2:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p Inj2'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`Inj3'':'a->extreal`,`C_Inj3:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p Inj3'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]

(*------------*)
THEN MP_TAC (Q.ISPECL [`HP'':'a->extreal`,`C_HP:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p HP'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`C'':'a->extreal`,`C_C:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p C'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`Bd'':'a->extreal`,`C_Bd:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p Bd'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*-----------------*)
THEN MP_TAC (Q.ISPECL [`IR'':'a->extreal`,`C_IR:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p IR'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`S'':'a->extreal`,`C_S:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p S'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]
(*------------*)
THEN MP_TAC (Q.ISPECL [`MT'':'a->extreal`,`C_MT:rat`,`s:rat`,`p:('a -> bool) # (('a -> bool) -> bool) # (('a -> bool) -> real)`,`m:num`,`n:num`] rel_med_lemma1)
THEN RW_TAC real_ss[]
THEN (`fail_event p MT'' (real_of_rat s) IN events p` by FULL_SIMP_TAC list_ss[two_dim_rel_event_list_def,rel_event_list_def])
THEN1 (RW_TAC std_ss[GSYM comp_rel_event_eq_fail_event]
      THEN DEP_REWRITE_TAC[EVENTS_COMPL]
      THEN RW_TAC std_ss[rel_event_def])
THEN FULL_SIMP_TAC std_ss[]);

(*-------------Porting to ACL2---------------*)

val rel_equiv_sexp = store_thm("rel_equiv_sexp",
  ``!m n s C_ER C_AC C_Ga C_EG C_Co C_DB C_CT C_CaA C_NS C_Inj1 C_Inj2 C_Inj3 C_HP C_C C_Bd C_IR C_S C_MT.  (s <> 0 /\ C_ER ≠ 0 ∧ C_AC ≠ 0 ∧ C_Ga ≠ 0 ∧ C_EG ≠ 0 ∧
      C_Co ≠ 0 ∧ C_DB ≠ 0 ∧ C_CT ≠ 0 ∧ C_CaA ≠ 0 ∧
      C_NS ≠ 0 ∧ C_Inj1 ≠ 0 ∧ C_Inj2 ≠ 0 ∧ C_Inj3 ≠ 0 ∧
      C_HP ≠ 0 ∧ C_C ≠ 0 ∧ C_Bd ≠ 0 ∧ C_IR ≠ 0 ∧
      C_S ≠ 0 ∧ C_MT ≠ 0) ==>
(rat ((1 −
 (1 − (expt_macl (C_ER * s) m n)) *
 ((1 − (expt_macl (C_AC * s) m n)) *
  (1 − (expt_macl (C_Ga * s) m n)))) *
((1 −
  (1 − (expt_macl (C_EG * s) m n)) *
  ((1 − (expt_macl (C_Co * s) m n)) *
   ((1 −  (expt_macl (C_DB * s) m n)) *
    ((1 −  (expt_macl (C_CT * s) m n)) *
     ((1 −  (expt_macl (C_CaA * s) m n)) *
      (1 −  (expt_macl (C_NS * s) m n))))))) *
 (1 −
  (1 −  (expt_macl (C_Inj1 * s) m n)) *
  ((1 −  (expt_macl (C_Inj2 * s) m n)) *
   ((1 −  (expt_macl (C_Inj3 * s) m n)) *
    ((1 −  (expt_macl (C_HP * s) m n)) *
     ((1 −  (expt_macl (C_C * s) m n)) *
      ((1 −  (expt_macl (C_Bd * s) m n)) *
       ((1 −  (expt_macl (C_IR * s) m n)) *
        ((1 −  (expt_macl (C_S * s) m n)) *
         (1 −  (expt_macl (C_MT * s) m n)))))))))))) = (mult
   (add (rat 1)
      (unary_minus
         (mult
            (add (rat 1)
               (unary_minus
                  (acl2_expt_macl (rat (C_ER * s)) (nat m) (nat n))))
            (mult
               (add (rat 1)
                  (unary_minus
                     (acl2_expt_macl (rat (C_AC * s)) (nat m) (nat n))))
               (add (rat 1)
                  (unary_minus
                     (acl2_expt_macl (rat (C_Ga * s)) (nat m)
                        (nat n))))))))
   (mult
      (add (rat 1)
         (unary_minus
            (mult
               (add (rat 1)
                  (unary_minus
                     (acl2_expt_macl (rat (C_EG * s)) (nat m) (nat n))))
               (mult
                  (add (rat 1)
                     (unary_minus
                        (acl2_expt_macl (rat (C_Co * s)) (nat m)
                           (nat n))))
                  (mult
                     (add (rat 1)
                        (unary_minus
                           (acl2_expt_macl (rat (C_DB * s)) (nat m)
                              (nat n))))
                     (mult
                        (add (rat 1)
                           (unary_minus
                              (acl2_expt_macl (rat (C_CT * s)) (nat m)
                                 (nat n))))
                        (mult
                           (add (rat 1)
                              (unary_minus
                                 (acl2_expt_macl (rat (C_CaA * s))
                                    (nat m) (nat n))))
                           (add (rat 1)
                              (unary_minus
                                 (acl2_expt_macl (rat (C_NS * s))
                                    (nat m) (nat n)))))))))))
      (add (rat 1)
         (unary_minus
            (mult
               (add (rat 1)
                  (unary_minus
                     (acl2_expt_macl (rat (C_Inj1 * s)) (nat m)
                        (nat n))))
               (mult
                  (add (rat 1)
                     (unary_minus
                        (acl2_expt_macl (rat (C_Inj2 * s)) (nat m)
                           (nat n))))
                  (mult
                     (add (rat 1)
                        (unary_minus
                           (acl2_expt_macl (rat (C_Inj3 * s)) (nat m)
                              (nat n))))
                     (mult
                        (add (rat 1)
                           (unary_minus
                              (acl2_expt_macl (rat (C_HP * s)) (nat m)
                                 (nat n))))
                        (mult
                           (add (rat 1)
                              (unary_minus
                                 (acl2_expt_macl (rat (C_C * s)) (nat m)
                                    (nat n))))
                           (mult
                              (add (rat 1)
                                 (unary_minus
                                    (acl2_expt_macl (rat (C_Bd * s))
                                       (nat m) (nat n))))
                              (mult
                                 (add (rat 1)
                                    (unary_minus
                                       (acl2_expt_macl (rat (C_IR * s))
                                          (nat m) (nat n))))
                                 (mult
                                    (add (rat 1)
                                       (unary_minus
                                          (acl2_expt_macl
                                             (rat (C_S * s)) (nat m)
                                             (nat n))))
                                    (add (rat 1)
                                       (unary_minus
                                          (acl2_expt_macl
                                             (rat (C_MT * s)) (nat m)
                                             (nat n)))))))))))))))) )``,
RW_TAC std_ss[RAT_THMS]
THEN DEP_REWRITE_TAC[expt_macl_equi_acl2_expt_macl]
THEN RW_TAC std_ss[ratTheory.RAT_NO_ZERODIV_NEG]
THEN RW_TAC std_ss[RAT_THMS]);




pr_sexp ``
   (add ((cpx 1 1 0 1))
      (unary_minus
         (mult
            (add (cpx 1 1 0 1)
               (unary_minus
                  (acl2_expt_macl ((cpx (5) 1 0 1)) (int (0)) (int (&10)))))
            (mult
               (add (cpx 1 1 0 1)
                  (unary_minus
                     (acl2_expt_macl ((cpx (6) 1 0 1)) (int 0) (int (&10)))))
               (add (cpx 1 1 0 1)
                  (unary_minus
                     (acl2_expt_macl ((cpx (7) 1 0 1)) (int 0)
                        (int (&10)))))))))``;

(* pr_sexp `` (add (1) 
       (unary_minus (
 add (1)  (unary_minus (acl2_expt_macl (rat (C_ER * s)) (nat m) (nat n))) *
 (add (1)  unary_minus (acl2_expt_macl (C_AC * s) m n)) *
  (1 − (acl2_expt_macl (C_Ga * s) m n)))))) *
((1 −
  (1 − (acl2_expt_macl (C_EG * s) m n)) *
  ((1 − (acl2_expt_macl (C_Co * s) m n)) *
   ((1 −  (acl2_expt_macl (C_DB * s) m n)) *
    ((1 −  (acl2_expt_macl (C_CT * s) m n)) *
     ((1 −  (acl2_expt_macl (C_CaA * s) m n)) *
      (1 −  (acl2_expt_macl (C_NS * s) m n))))))) *
 (1 −
  (1 −  (acl2_expt_macl (C_Inj1 * s) m n)) *
  ((1 −  (acl2_expt_macl (C_Inj2 * s) m n)) *
   ((1 −  (acl2_expt_macl (C_Inj3 * s) m n)) *
    ((1 −  (acl2_expt_macl (C_HP * s) m n)) *
     ((1 −  (acl2_expt_macl (C_C * s) m n)) *
      ((1 −  (acl2_expt_macl (C_Bd * s) m n)) *
       ((1 −  (acl2_expt_macl (C_IR * s) m n)) *
        ((1 −  (acl2_expt_macl (C_S * s) m n)) *
         (1 −  (acl2_expt_macl (C_MT * s) m n))))))))))))``;
*)